@@ -1,23 +1,39 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
+#ifndef GKRELLMD_H
+#define GKRELLMD_H
+
+#include "log.h"
 
 #include <glib.h>
 
@@ -33,29 +49,27 @@
 #endif
 
 #if !defined(WIN32)
-#include <unistd.h>
-#include <utime.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/ioctl.h>
-#include <pwd.h>
-#include <grp.h>
-#if defined(__solaris__)
-#include <sys/filio.h>
-#endif
-#include <sys/select.h>
-#include <sys/wait.h>
+	#include <unistd.h>
+	#include <utime.h>
+	#include <sys/socket.h>
+	#include <sys/time.h>
+	#include <netinet/in.h>
+	#include <arpa/inet.h>
+	#include <netdb.h>
+	#include <sys/ioctl.h>
+	#include <pwd.h>
+	#include <grp.h>
+	#if defined(__solaris__)
+		#include <sys/filio.h>
+	#endif /* defined(__solaris__) */
+	#include <sys/select.h>
+	#include <sys/wait.h>
 #else
-#define SIGTSTP SIGTERM
-#define ioctl ioctlsocket
-#include "../src/win32.h"
-#if defined(_WINDOWS)
-#include "resource.h"
-#endif
-#endif
+	#include <winsock2.h>
+	#include <ws2tcpip.h>
+	typedef int sa_family_t; // WIN32 uses int for ai_family;
+	#include <stdint.h> // defines uint32_t
+#endif /* !defined(WIN32) */
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -86,14 +100,14 @@
 #   define gettext(String) (String)
 #   define dgettext(Domain,String) (String)
 #   define dcgettext(Domain,String,Type) (String)
-#   define bindtextdomain(Domain,Directory) (Domain) 
+#   define bindtextdomain(Domain,Directory) (Domain)
 #endif  /* ENABLE_NLS */
 
 /* -------------------------------------------------------------------
 */
 #define GKRELLMD_VERSION_MAJOR   2
-#define GKRELLMD_VERSION_MINOR   2
-#define GKRELLMD_VERSION_REV     9
+#define GKRELLMD_VERSION_MINOR   3
+#define GKRELLMD_VERSION_REV     4
 #define GKRELLMD_EXTRAVERSION    ""
 
 #define GKRELLMD_CHECK_VERSION(major,minor,rev)    \
@@ -102,18 +116,23 @@
 (GKRELLMD_VERSION_MAJOR == (major) && GKRELLMD_VERSION_MINOR == (minor) && \
 GKRELLMD_VERSION_REV >= (rev)))
 
-#define GKRELLMD_CONFIG     "gkrellmd.conf"
+#define GKRELLMD_CONFIG				"gkrellmd.conf"
+#if defined(WIN32)
+	// no dot in front of config-filename on win32
+	#define GKRELLMD_USER_CONFIG  GKRELLMD_CONFIG
+#else
+	#define GKRELLMD_USER_CONFIG	".gkrellmd.conf"
+#endif
 
 #define GKRELLMD_PLUGINS_DIR		".gkrellm2/plugins-gkrellmd"
-#define GKRELLMD_LOCAL_PLUGINS_DIR	"/usr/local/lib/gkrellm2/plugins-gkrellmd"
-#if !defined(GKRELLMD_SYSTEM_PLUGINS_DIR)
-#define GKRELLMD_SYSTEM_PLUGINS_DIR	"/usr/lib/gkrellm2/plugins-gkrellmd"
-#endif
-
 #if !defined(WIN32)
-#define	GKRELLMD_SYS_ETC	"/etc"
-#define GKRELLMD_LOCAL_ETC	"/usr/local/etc"
-#endif
+	#define GKRELLMD_LOCAL_PLUGINS_DIR	"/usr/local/lib/gkrellm2/plugins-gkrellmd"
+	#if !defined(GKRELLMD_SYSTEM_PLUGINS_DIR)
+		#define GKRELLMD_SYSTEM_PLUGINS_DIR	"/usr/lib/gkrellm2/plugins-gkrellmd"
+	#endif
+	#define GKRELLMD_SYS_ETC	"/etc"
+	#define GKRELLMD_LOCAL_ETC	"/usr/local/etc"
+#endif // !defined(WIN32)
 
 
 typedef struct _GkrellmdClient
@@ -223,7 +242,8 @@ gint		gkrellmd_send_to_client(GkrellmdClient *client, gchar *buf);
   /* Misc
   */
 void		gkrellmd_add_mailbox(gchar *);
-
+GkrellmdTicks *gkrellmd_ticks(void);
+gint		gkrellmd_get_timer_ticks(void);
 
 
 #if !GLIB_CHECK_VERSION(2,0,0)
@@ -251,3 +271,4 @@ gchar		*g_path_get_basename(gchar *file_name);
 
 #endif
 
+#endif // GKRELLMD_H
