@@ -1,25 +1,36 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
-
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 
@@ -508,6 +519,7 @@ gkrellm_alert_set_triggers(GkrellmAlert *alert,
 	alert->check_hardwired = TRUE;
 	}
 
+
 void
 gkrellm_alert_commands_config(GkrellmAlert *alert,
 						gboolean alarm, gboolean warn)
@@ -724,8 +736,12 @@ void
 gkrellm_alert_get_alert_state(GkrellmAlert *alert, gboolean *alarm_state,
 				gboolean *warn_state)
 	{
-	*warn_state = (alert->high.warn_on || alert->low.warn_on);
-	*alarm_state = (alert->high.alarm_on || alert->low.alarm_on);
+	if (!alert)
+		return;
+	if (warn_state)
+		*warn_state = (alert->high.warn_on || alert->low.warn_on);
+	if (alarm_state)
+		*alarm_state = (alert->high.alarm_on || alert->low.alarm_on);
 	}
 
 
