@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -206,13 +218,17 @@ add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
 	}
 
 static void
-disk_assign_data(DiskMon *disk, guint64 rb, guint64 wb)
+disk_assign_data(DiskMon *disk, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	if (!disk)
 		return;
 	disk->rb = rb;
 	disk->wb = wb;
-	if (disk->subdisk == -1)	/* not a subdisk (partition) */
+
+	/* Add data to composite disk if this is not a subdisk (partition) and
+	|  not a virtual disk (eg /dev/mdX software RAID multi-disk).
+	*/
+	if (disk->subdisk == -1 && !virtual)
 		{
 		composite_disk->rb += rb;
 		composite_disk->wb += wb;
@@ -258,10 +274,10 @@ gkrellm_disk_units_are_blocks(void)
 	{
 	units_are_blocks = TRUE;
 	}
-	
+
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
-			guint64 rb, guint64 wb)
+			guint64 rb, guint64 wb, gboolean virtual)
 	{
 	DiskMon	*disk;
 	gchar	*name;
@@ -275,11 +291,11 @@ gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 		if (name)
 			disk = add_disk(name, name, device_number, unit_number, order);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 	{
 	DiskMon	*disk;
 	gchar	name[32], label[32];
@@ -293,11 +309,12 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb)
 		sprintf(label, "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, label, 0, 0, n);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
-gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
+gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb,
+			gboolean virtual)
 	{
 	DiskMon	*disk;
 	gint	order = -1;
@@ -312,7 +329,7 @@ gkrellm_disk_assign_data_by_name(gchar *name, guint64 rb, guint64 wb)
 			order = (*order_from_name)(name);
 		disk = add_disk(name, name, 0, 0, order);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, virtual);
 	}
 
 void
@@ -338,7 +355,7 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 			return;
 		disk = add_subdisk(subdisk_name, disk_name, subdisk);
 		}
-	disk_assign_data(disk, rb, wb);
+	disk_assign_data(disk, rb, wb, FALSE);
 	}
 
 
@@ -390,6 +407,8 @@ format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 				blocks = r_blocks;
 			else if (c == 'w')
 				blocks = w_blocks;
+			else if (c == 'L')
+				len = snprintf(buf, size, "%s", disk->name);
 			else if (c == 'H')
 				len = snprintf(buf, size, "%s", gkrellm_sys_get_host_name());
 			else
@@ -411,7 +430,7 @@ format_disk_data(DiskMon *disk, gchar *src_string, gchar *buf, gint size)
 		size -= len;
 		buf += len;
 		}
-	*buf = '\0';	
+	*buf = '\0';
 	}
 
 static void
@@ -419,7 +438,7 @@ draw_disk_extra(DiskMon *disk)
 	{
 	gchar	buf[128];
 
-	if (!disk->extra_info)
+	if (!disk->chart || !disk->extra_info)
 		return;
 	format_disk_data(disk, text_format_locale, buf, sizeof(buf));
 	if (!disk->new_text_format)
@@ -432,7 +451,8 @@ static void
 cb_command_process(GkrellmAlert *alert, gchar *src, gchar *dst, gint len,
 			DiskMon *disk)
 	{
-	format_disk_data(disk, src, dst, len);
+	if (disk->chart)
+		format_disk_data(disk, src, dst, len);
 	}
 
 static void
@@ -487,7 +507,7 @@ gkrellm_disk_temperature_display(gpointer sr, gchar *id_name, gfloat t,
 		{
 		disk = (DiskMon *) list->data;
 		if (   strncmp(disk->name, disk_name, len)
-			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+			|| (disk->name[len] != '\0' && !isdigit((unsigned char)disk->name[len]))
 		   )
 			continue;
 		if (!disk->enabled || !disk->chart || !disk->chart->panel->decal_list)
@@ -548,7 +568,7 @@ gkrellm_disk_temperature_remove(gchar *id_name)
 		if (!disk->chart)
 			continue;
 		if (   strncmp(disk->name, disk_name, len)
-			|| (disk->name[len] != '\0' && !isdigit(disk->name[len]))
+			|| (disk->name[len] != '\0' && !isdigit((unsigned char)disk->name[len]))
 		   )
 			continue;
 		p = disk->chart->panel;
@@ -904,7 +924,7 @@ save_disk_config(FILE *f)
 		|  a disk saved into the config here should be artificially created
 		|  should it not be present at next gkrellm startup.
 		*/
-		have_enabled_subdisks = (disk->subdisk == -1) ? 
+		have_enabled_subdisks = (disk->subdisk == -1) ?
 					any_enabled_subdisks(list->next) : FALSE;
 
 		fprintf(f, "%s device %s %d %d %d %d %d %d %d\n",
@@ -1028,7 +1048,7 @@ static GtkTreeRowReference	*row_reference;
 static GtkTreeSelection		*selection;
 
 static GtkWidget			*launch_vbox,
-							*text_format_combo;
+							*text_format_combo_box;
 
 static GtkWidget			*alert_button;
 
@@ -1265,7 +1285,7 @@ cb_enable(GtkCellRendererText *cell, gchar *path_string, gpointer data)
 				DISK_COLUMN, &disk,
 				-1);
 	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
-				ENABLE_COLUMN, !enabled, 
+				ENABLE_COLUMN, !enabled,
 				-1);
 	if (enabled)
 		{
@@ -1295,8 +1315,10 @@ cb_text_format(GtkWidget *widget, gpointer data)
 	GList	*list;
 	DiskMon	*disk;
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = disk_mon_list; list; list = list->next)
 		{
@@ -1313,6 +1335,7 @@ static gchar	*disk_info_text[] =
 {
 N_("<h>Chart Labels\n"),
 N_("Substitution variables for the format string for chart labels:\n"),
+N_("\t$L    the Disk label\n"),
 N_("\t$M    maximum chart value\n"),
 N_("\t$T    total read bytes + write bytes\n"),
 N_("\t$r    read bytes\n"),
@@ -1390,30 +1413,29 @@ create_disk_tab(GtkWidget *tab_vbox)
 
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
-	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 350, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list, "\\c\\f$M\\n$T");
-	list = g_list_append(list, "\\c\\f$M\\b$T");
-	list = g_list_append(list,
-					_("\\f\\ww\\r\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
-			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo_box, FALSE, FALSE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		text_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\n$T");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\b$T");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		_("\\f\\ww\\r\\f$M\\D2\\f\\ar\\. $r\\D1\\f\\aw\\. $w"));
+	gtk_combo_box_set_active(GTK_COMBO_BOX(text_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-						4, TRUE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox, NULL,
 						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_show(launch_vbox);
