@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -242,11 +254,12 @@ draw_sysname(void)
 void
 update_host(void)
 	{
-	gchar			**parts;
-	gint			connect_state, delta, step, h_scroll;
-	gint			hz = gkrellm_update_HZ();
-	static gint		reconnect_timeout, y_target, asym;
-	static gboolean	recheck_sysname;
+	gchar						**parts;
+	gint						delta, step, h_scroll;
+	gint						hz = gkrellm_update_HZ();
+	static gint					reconnect_timeout, y_target, asym;
+	static gboolean				recheck_sysname;
+	enum GkrellmConnectState	connect_state;
 
 	if (decal_sysname && system_name_mode == 1 && system_dtext.reduced)
 		{
@@ -283,7 +296,7 @@ update_host(void)
 	/* If we loose the server connection, trigger a hardwired alarm.
 	*/
 	connect_state = gkrellm_client_server_connect_state();
-	if (connect_state == 2)		/* thread is trying a reconnect */
+	if (connect_state == CONNECTING)		/* thread is trying a reconnect */
 		{
 		if (_GK.client_server_reconnect_timeout <= 0)
 			{
@@ -291,7 +304,7 @@ update_host(void)
 			draw_sysname();
 			}
 		}
-	else if (connect_state == 0)	/* Lost connection			*/
+	else if (connect_state == DISCONNECTED)	/* Lost connection			*/
 		{
 		gkrellm_check_alert(server_alert, 2.0);		/* Alarm 	*/
 		if (   _GK.client_server_reconnect_timeout > 0
