@@ -1,24 +1,36 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
- 
+
 #include "gkrellm.h"
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
@@ -154,6 +166,13 @@ _log_active_port(ActiveTCP *tcp)
 	tcp->new_hit = 1;
 	new_tcp = _tcp_alloc();
 	*new_tcp = *tcp;
+
+	if (_GK.debug_level & DEBUG_INET)
+		{
+		ap = inet_ntoa(tcp->remote_addr);
+		g_print("inet  OO----->  %x %s:%x\n",
+				tcp->local_port, ap, tcp->remote_port);
+		}
 	active_tcp_list = g_list_prepend(active_tcp_list, (gpointer) new_tcp);
 	return new_tcp;		/* A new hit	*/
 	}
@@ -305,6 +324,8 @@ format_chart_text(InetMon *in, gchar *src_string, gchar *buf, gint size)
 						value += gkrellm_get_chartdata_data(cd, w - n);
 					}
 				}
+			else if (c == 'H')
+				s1 = gkrellm_sys_get_host_name();
 			if (value >= 0)
 				len = snprintf(buf, size, "%d", value);
 			else
@@ -316,7 +337,7 @@ format_chart_text(InetMon *in, gchar *src_string, gchar *buf, gint size)
 		size -= len;
 		buf += len;
 		}
-	*buf = '\0';	
+	*buf = '\0';
 	}
 
 static void
@@ -502,7 +523,7 @@ update_inet(void)
 	GkrellmChart	*cp;
 	ActiveTCP		*tcp;
 	GList			*list;
-	gchar			buf[32];
+	gchar			buf[32], *ap;
 	gint			i;
 	static gint		check_tcp;
 
@@ -536,6 +557,12 @@ update_inet(void)
 				if (list == active_tcp_list)
 					active_tcp_list = active_tcp_list->next;
 				list = g_list_remove(list, tcp);
+				if (_GK.debug_level & DEBUG_INET)
+					{
+					ap = inet_ntoa(tcp->remote_addr);
+					g_print("inet  XX----->  %x %s:%x\n",
+							tcp->local_port, ap, tcp->remote_port);
+					}
 				free_tcp_list = g_list_prepend(free_tcp_list, tcp);
 				}
 			else
@@ -633,6 +660,7 @@ get_connection_string_thread(void *data)
 	struct hostent			*hostent;
 	gchar					buf[64];
 #endif
+	gchar					*remote_host, *udp_note;
 
 	if (in->connection_string)
 		in->connection_string = g_string_truncate(in->connection_string, 0);
@@ -648,8 +676,7 @@ get_connection_string_thread(void *data)
 			case AF_INET:
 				sin = (struct sockaddr_in *)&ss;
 				salen = sizeof(struct sockaddr_in);
-				memcpy(&sin->sin_addr,
-						&tcp->remote_addr, salen);
+				memcpy(&sin->sin_addr, &tcp->remote_addr, salen);
 #if defined(SIN6_LEN)
 				sin->sin_len = salen;
 #endif
@@ -658,8 +685,7 @@ get_connection_string_thread(void *data)
 			case AF_INET6:
 				sin6 = (struct sockaddr_in6 *)&ss;
 				salen = sizeof(struct sockaddr_in6);
-				memcpy(&sin6->sin6_addr,
-						&tcp->remote_addr6, salen);
+				memcpy(&sin6->sin6_addr, &tcp->remote_addr6, salen);
 #if defined(SIN6_LEN)
 				sin6->sin6_len = salen;
 #endif
@@ -673,20 +699,19 @@ get_connection_string_thread(void *data)
 		if (getnameinfo((struct sockaddr *)&ss, salen,
 					hbuf, sizeof(hbuf), NULL, 0, flag))
 			continue;
-		snprintf(buf, sizeof(buf), "%6d:  %s\n",
-					tcp->local_port, hbuf);
+		remote_host = hbuf;
 #else
 		hostent = gethostbyaddr((char *) &tcp->remote_addr,
 					sizeof(struct in_addr), AF_INET);
 		if (hostent)
-			snprintf(buf, sizeof(buf),
-				 "%6d:  %s\n", tcp->local_port,
-				 hostent->h_name);
+			remote_host = hostent->h_name;
 		else
-			snprintf(buf, sizeof(buf),
-				 "%6d:  %s\n", tcp->local_port,
-				 inet_ntoa(tcp->remote_addr));
+			remote_host = inet_ntoa(tcp->remote_addr);
 #endif
+		udp_note = tcp->is_udp ? " (UDP)" : "";
+		snprintf(buf, sizeof(buf), "%6d:  %s%s\n",
+				tcp->local_port, remote_host, udp_note);
+
 		g_string_append(in->connection_string, buf);
 		}
 	if (in->connection_string->len == 0)
@@ -1067,7 +1092,7 @@ save_inet_config(FILE *f)
 		}
 	fprintf(f, "%s text_format all %s\n", INET_CONFIG_KEYWORD, text_format);
 	if (!_GK.client_mode)
-		fprintf(f, "%s update_interval %d\n",
+		fprintf(f, "%s update_interval all %d\n",
 					INET_CONFIG_KEYWORD, update_interval);
 	}
 
@@ -1396,7 +1421,7 @@ static GtkWidget	*launch_vbox;
 static GtkWidget	*data0_range_button,
 					*data1_range_button;
 
-static GtkWidget	*text_format_combo;
+static GtkWidget	*text_format_combo_box;
 
 
 static void
@@ -1691,7 +1716,7 @@ cb_enter(GtkWidget *widget, gpointer data)
 		in->port1_0 = atoi(gkrellm_gtk_entry_get_text(&port1_0_entry));
 		in->port1_1 = atoi(gkrellm_gtk_entry_get_text(&port1_1_entry));
 		}
-	
+
 	/* Validate the values
 	*/
 	if (   (!*(in->label0) && !*(in->label1))
@@ -1744,8 +1769,10 @@ cb_text_format(GtkWidget *widget, gpointer data)
 	{
 	GList   *list;
 	gchar   *s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	for (list = inet_mon_list; list; list = list->next)
 		draw_inet_chart((InetMon *) list->data);
@@ -1815,7 +1842,7 @@ create_inet_tab(GtkWidget *tab_vbox)
 	GList			*list;
 	InetMon			*in;
 	gint			i;
-	
+
 	row_reference = NULL;
 
 	tabs = gtk_notebook_new();
@@ -1986,30 +2013,30 @@ create_inet_tab(GtkWidget *tab_vbox)
             		cb_update_interval, NULL, FALSE,
             		_("Seconds between updates"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-				4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
-	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 350, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list,
-				"\\r\\f $M\\t$a\\f $l\\N$A\\f $L");
-	list = g_list_append(list,
-				"\\r\\f $M\\D1$a\\f $l\\D2$A\\f $L");
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
+	label = gtk_label_new("");	/* padding */
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
+
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo_box, FALSE, FALSE, 2);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
 			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+			DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+			"\\r\\f $M\\t$a\\f $l\\N$A\\f $L");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+			"\\r\\f $M\\D1$a\\f $l\\D2$A\\f $L");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(text_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, TRUE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_show(launch_vbox);
@@ -2068,5 +2095,5 @@ gkrellm_init_inet_monitor(void)
 		inet_data_dir = gkrellm_make_data_file_name("inet", NULL);
 		return &monitor_inet;
 		}
-	return FALSE;
+	return NULL;
 	}
