@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2006 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -192,9 +204,10 @@ force_meminfo_update(void)
 static gint
 format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 	{
-	gulong	t, u, f;
-	gchar	*s, *label;
-	gint	len;
+	gulong		t, u, f, ur, fr;
+	gchar		*s, *label;
+	gint		len;
+	gboolean	raw;
 
 	if (!buf || size < 1)
 		return -1;
@@ -207,11 +220,17 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 	u = MEG(mm->used);
 	f = t - u;
 
+	fr = MEG(mem.free);
+	ur = u + MEG(mem.shared + mem.buffers + mem.cached);
+
 	for (s = src_string; *s != '\0' && size > 0; ++s)
 		{
 		len = 1;
+		raw = FALSE;
 		if (*s == '$' && *(s + 1) != '\0')
 			{
+			if (*(s + 2) == 'r')  /* print raw free/used */
+				raw = TRUE;
 			switch(*(s + 1))
 				{
 				case 'l':
@@ -221,18 +240,20 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 					len = snprintf(buf, size, "%ldM", t);
 					break;
 				case 'u':
-					len = snprintf(buf, size, "%ldM", u);
+					len = snprintf(buf, size, "%ldM", raw ? ur : u);
 					break;
 				case 'U':
 					if (t > 0)
-						len = snprintf(buf, size, "%ld%%", 100 * u / t);
+						len = snprintf(buf, size, "%ld%%",
+							100 * (raw ? ur : u) / t);
 					break;
 				case 'f':
-					len = snprintf(buf, size, "%ldM", f);
+					len = snprintf(buf, size, "%ldM", raw ? fr : f);
 					break;
 				case 'F':
 					if (t > 0)
-						len = snprintf(buf, size, "%ld%%", 100 * f / t);
+						len = snprintf(buf, size, "%ld%%",
+							100 * (raw ? fr : f) / t);
 					break;
 				case 's':
 					if (mm == &mem)
@@ -260,6 +281,8 @@ format_meminfo_data(MeminfoMeter *mm, gchar *src_string, gchar *buf, gint size)
 					break;
 				}
 			++s;
+			if (raw)
+				++s;
 			}
 		else
 			*buf = *s;
@@ -382,7 +405,7 @@ format_chart_text(MeminfoChart *mc, gchar *buf, gint size)
 		size -= len;
 		buf += len;
 		}
-	*buf = '\0';	
+	*buf = '\0';
 	}
 
 static void
@@ -1081,10 +1104,10 @@ static GtkWidget	*mem_launch_entry,
 					*swap_launch_entry,
 					*swap_tooltip_entry;
 
-static GtkWidget	*mem_format_combo,
-					*swap_format_combo;
+static GtkWidget	*mem_format_combo_box,
+					*swap_format_combo_box;
 
-static GtkWidget	*text_format_combo;
+static GtkWidget	*text_format_combo_box;
 
 static GtkWidget	*mem_alert_button,
 					*swap_alert_button;
@@ -1094,8 +1117,10 @@ static void
 cb_text_format(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(text_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(text_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 	gkrellm_locale_dup_string(&text_format, s, &text_format_locale);
 	refresh_chart(&swap_chart);
 	}
@@ -1170,8 +1195,10 @@ static void
 cb_mem_format(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(mem_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(mem_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	/* In case Pango markup tags, don't accept line unless valid markup.
 	|  Ie, markup like <span ...> xxx </span> or <b> xxx </b>
@@ -1190,8 +1217,10 @@ static void
 cb_swap_format(GtkWidget *widget, gpointer data)
 	{
 	gchar	*s;
+	GtkWidget *entry;
 
-	s = gkrellm_gtk_entry_get_text(&(GTK_COMBO(swap_format_combo)->entry));
+	entry = gtk_bin_get_child(GTK_BIN(swap_format_combo_box));
+	s = gkrellm_gtk_entry_get_text(&entry);
 
 	if (   strchr(s, '<') != NULL
 		&& !pango_parse_markup(s, -1, 0, NULL, NULL, NULL, NULL)
@@ -1246,6 +1275,9 @@ N_("\t$s    shared MiB\n"),
 N_("\t$b    buffered MiB\n"),
 N_("\t$c    cached MiB\n"),
 "\n",
+N_("The free and used variables may have a 'r' qualifier for printing\n"
+   "raw free and raw used values.  For example: $fr for raw free.\n"),
+"\n",
 N_("Substitution variables may be used in alert commands.\n"),
 
 "\n",
@@ -1265,7 +1297,6 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	GtkWidget		*table;
 	GtkWidget		*hbox;
 	GtkWidget		*text, *label;
-	GList			*list;
 	gint			i;
 
 	tabs = gtk_notebook_new();
@@ -1275,7 +1306,9 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 /* --Options Tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Options"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Swap"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Swap"),
+				4, 0, TRUE);
     gkrellm_gtk_check_button_connected(vbox1, NULL,
 				swap_chart.enabled, FALSE, FALSE, 0,
 				cb_swap_chart_enable, NULL,
@@ -1291,7 +1324,9 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 	if (!swap.enabled)
 		gtk_widget_set_sensitive(swap_alert_button, FALSE);
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Memory"), 4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Memory"),
+				4, 0, TRUE);
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
     gkrellm_gtk_check_button_connected(hbox, NULL,
@@ -1310,75 +1345,78 @@ create_meminfo_tab(GtkWidget *tab_vbox)
 /* -- Setup tab */
 	vbox = gkrellm_gtk_framed_notebook_page(tabs, _("Setup"));
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Chart Labels"),
-			 4, FALSE, 0, 2);
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 5);
-	text_format_combo = gtk_combo_new();
-	gtk_widget_set_size_request (GTK_WIDGET(text_format_combo), 300, -1);
-	gtk_box_pack_start(GTK_BOX(hbox), text_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, text_format);
-	list = g_list_append(list, DEFAULT_TEXT_FORMAT);
-	list = g_list_append(list, "$T\\C\\f$M");
-	list = g_list_append(list, "\\c\\f$M\\b$T");
-	list = g_list_append(list,
-				"\\ww\\C\\f$M\\D2\\f\\ai\\.$i\\D1\\f\\ao\\.$o");
-	list = g_list_append(list,
-				"\\ww\\C\\f$M\\D3\\f\\ai\\.$i\\D0\\f\\ao\\.$o");
-	gtk_combo_set_popdown_strings(GTK_COMBO(text_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(text_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(text_format_combo)->entry),
-			text_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(text_format_combo)->entry), "changed",
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Chart Labels"),
+				4, 0, TRUE);
+	text_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(vbox1), text_format_combo_box, FALSE, FALSE, 2);
+
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		text_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		DEFAULT_TEXT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"$T\\C\\f$M");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\c\\f$M\\b$T");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\ww\\C\\f$M\\D2\\f\\ai\\.$i\\D1\\f\\ao\\.$o");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(text_format_combo_box),
+		"\\ww\\C\\f$M\\D3\\f\\ai\\.$i\\D0\\f\\ao\\.$o");
+	gtk_combo_box_set_active(GTK_COMBO_BOX(text_format_combo_box), 0);
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(text_format_combo_box)), "changed",
 			G_CALLBACK(cb_text_format), NULL);
 
-	vbox1 = gkrellm_gtk_framed_vbox(vbox, _("Format String for Panel Labels"),
-			4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Format String for Panel Labels"),
+				4, 6, TRUE);
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
-	mem_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), mem_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, mem.data_format);
-	list = g_list_append(list, DEFAULT_FORMAT);
-	list = g_list_append(list, _("$t - $u used"));
-	list = g_list_append(list, _("$t - $U"));
-	list = g_list_append(list, _("$t - $u used  $s sh  $b bf  $c ca"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(mem_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(mem_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mem_format_combo)->entry),
+	mem_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(hbox), mem_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mem_format_combo_box),
+		mem.data_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mem_format_combo_box),
+		DEFAULT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mem_format_combo_box),
+		_("$t - $u used"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mem_format_combo_box),
+		_("$t - $U"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(mem_format_combo_box),
+		_("$t - $u used  $s sh  $b bf  $c ca"));
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(mem_format_combo_box))),
 			mem.data_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(mem_format_combo)->entry), "changed",
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(mem_format_combo_box)), "changed",
 			G_CALLBACK(cb_mem_format), NULL);
 	label = gtk_label_new(_("Mem"));
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
-	swap_format_combo = gtk_combo_new();
-	gtk_box_pack_start(GTK_BOX(hbox), swap_format_combo, FALSE, TRUE, 0);
-	list = NULL;
-	list = g_list_append(list, swap.data_format);
-	list = g_list_append(list, DEFAULT_FORMAT);
-	list = g_list_append(list, _("$t - $u used"));
-	list = g_list_append(list, _("$t - $U"));
-	gtk_combo_set_popdown_strings(GTK_COMBO(swap_format_combo), list);
-	gtk_combo_set_case_sensitive(GTK_COMBO(swap_format_combo), TRUE);
-	g_list_free(list);
-	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(swap_format_combo)->entry),
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 0);
+	swap_format_combo_box = gtk_combo_box_entry_new_text();
+	gtk_box_pack_start(GTK_BOX(hbox), swap_format_combo_box, TRUE, TRUE, 0);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(swap_format_combo_box),
+		swap.data_format);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(swap_format_combo_box),
+		DEFAULT_FORMAT);
+	gtk_combo_box_append_text(GTK_COMBO_BOX(swap_format_combo_box),
+		_("$t - $u used"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(swap_format_combo_box),
+		_("$t - $U"));
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(swap_format_combo_box))),
 			swap.data_format);
-	g_signal_connect(G_OBJECT(GTK_COMBO(swap_format_combo)->entry), "changed",
+	g_signal_connect(G_OBJECT(GTK_COMBO_BOX(swap_format_combo_box)), "changed",
 			G_CALLBACK(cb_swap_format), NULL);
 	label = gtk_label_new(_("Swap"));
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
 
-	vbox1 = gkrellm_gtk_framed_vbox_end(vbox, _("Launch Commands"),
-				4, FALSE, 0, 2);
+	vbox1 = gkrellm_gtk_category_vbox(vbox,
+				_("Launch Commands"),
+				4, 0, TRUE);
 	table = gkrellm_gtk_launcher_table_new(vbox1, 2);
 	gkrellm_gtk_config_launcher(table, 0,  &mem_launch_entry,
 				&mem_tooltip_entry, _("Mem"), &(mem.launch));
