@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -68,9 +80,15 @@
 #include "sysdeps/gtop.c"
 #endif
 
-#include <sys/utsname.h>
+#if defined(WIN32)
+#include "sysdeps/win32.c"
+#endif
 
-#ifndef SENSORS_COMMON
+#if !defined(WIN32)
+#include <sys/utsname.h>
+#endif
+
+#if !defined(SENSORS_COMMON) && !defined(WIN32)
 static gboolean (*mbmon_check_func)();
 #endif
 
@@ -86,6 +104,7 @@ gkrellm_sys_get_host_name(void)
 	return buf;
 	}
 
+#if !defined(WIN32)
 gchar *
 gkrellm_sys_get_system_name(void)
 	{
@@ -98,12 +117,13 @@ gkrellm_sys_get_system_name(void)
 		sname = g_strdup("unknown name");
 	return sname;
 	}
+#endif
 
 gboolean
 gkrellm_sys_sensors_mbmon_port_change(gint port)
 	{
 	gboolean	result = FALSE;
-
+#if !defined(WIN32)
 	_GK.mbmon_port = port;
 
 	/* mbmon_check_func will be set if sysdep code has included
@@ -116,11 +136,16 @@ gkrellm_sys_sensors_mbmon_port_change(gint port)
 		gkrellm_sensors_model_update();
 		gkrellm_sensors_rebuild(TRUE, TRUE, TRUE);
 		}
+#endif
 	return result;
 	}
 
 gboolean
 gkrellm_sys_sensors_mbmon_supported(void)
 	{
+#if !defined(WIN32)
 	return mbmon_check_func ? TRUE : FALSE;
+#else
+	return FALSE;
+#endif
 	}
