@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2009 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -216,7 +228,7 @@ net_wm_state(gchar *hint, gboolean state)
 	xev.xclient.data.l[2] = 0;
 
 	XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(),
-				False, SubstructureNotifyMask, &xev);
+				False, SubstructureNotifyMask|SubstructureRedirectMask, &xev);
 	}
 
 void
@@ -255,11 +267,17 @@ static FILE		*f_lock;
 static gboolean
 _gkrellm_get_lock(void)
 	{
-	gchar	*lock_file, *display, *s;
+	gchar	*lock_dir, *lock_file, *display, *s;
+	gchar	buf[32];
+
+	snprintf(buf, sizeof(buf), "LCK..%d", (gint) getuid());
 
 #if defined(F_TLOCK)
-	lock_file = gkrellm_make_config_file_name(gkrellm_homedir(),
-					GKRELLM_LOCK_FILE);
+	lock_dir = "/var/lock/gkrellm";
+	if (!g_file_test(lock_dir, G_FILE_TEST_IS_DIR))
+		mkdir(lock_dir, 0755);
+
+	lock_file = gkrellm_make_config_file_name(lock_dir, buf);
 	display = XDisplayName(NULL);
 	if (display)
 		{
@@ -274,6 +292,11 @@ _gkrellm_get_lock(void)
 		&& errno == EAGAIN
 	   )
 		return FALSE;
+	if (f_lock)
+		{
+		fprintf(f_lock, "%10d\n", (gint) getpid());
+		fflush(f_lock); 
+		}
 #endif
 	return TRUE;
 	}
@@ -444,7 +467,8 @@ gkrellm_winop_place_gkrellm(gchar *geom)
 	gint	place, x, y, w_gkrell, h_gkrell;
 
 	x = y = 0;
-	place = XParseGeometry(geom, &x, &y, &w_gkrell, &h_gkrell);
+	place = XParseGeometry(geom, &x, &y,
+				(guint *) &w_gkrell, (guint *) &h_gkrell);
 
 	w_gkrell = _GK.chart_width + _GK.frame_left_width + _GK.frame_right_width;
 	h_gkrell = _GK.monitor_height + _GK.total_frame_height;
@@ -514,7 +538,7 @@ gkrellm_winop_draw_rootpixmap_onto_transparent_chart(GkrellmChart *cp)
 	GkrellmMargin	*m;
 	gint			x, y;
 
-	if (   root_xpixmap == None || !cp->transparency
+	if (   root_xpixmap == None || trans_gc == NULL || !cp->transparency
 		|| !cp->drawing_area || !cp->drawing_area->window
 	   )
 		return FALSE;
@@ -581,7 +605,7 @@ gkrellm_winop_draw_rootpixmap_onto_transparent_panel(GkrellmPanel *p)
 	Window	child;
 	gint	x, y;
 
-	if (   root_xpixmap == None || !p->transparency
+	if (   root_xpixmap == None || trans_gc == NULL || !p->transparency
 		|| !p->drawing_area || !p->drawing_area->window
 	   )
 		return FALSE;
@@ -665,8 +689,8 @@ gkrellm_winop_apply_rootpixmap_transparency(void)
 	gulong			nitems, bytes_after;
 	gint			depth_visual;
 	Window			root_return;
-	guint			x_ret, y_ret, w_ret, h_ret, bw_ret, depth_ret;
-	gint			x_root, y_root;
+	guint			w_ret, h_ret, bw_ret, depth_ret;
+	gint			x_ret, y_ret, x_root, y_root;
 
 	if (!_GK.any_transparency)
 		return;
